/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, ViewContainerRef} from '@angular/core';
import {RepetierServer, ConnectionHandler, ServerAction, PrintersState, PrinterState, ServerState,WifiState,RepetierState} from '@repetier/binding';
import {RSModal} from "@repetier/front-ui";
import {select,NgRedux} from 'ng2-redux';
import {Observable} from 'rxjs/Observable';
import {ISubscription} from "rxjs/Subscription";
import {Router} from "@angular/router";

@Component({
    selector: 'my-app',
    template: `<router-outlet></router-outlet>`
})
export class AppComponent {
    @select('printers') printers$: Observable<PrintersState>;
    @select('server') server$:Observable<ServerState>;
    @select('wifi') wifi$:Observable<WifiState>;
    private printersSubscription: ISubscription;
    private serverSubscription: ISubscription;
    private wifiSubscription: ISubscription;
    printer: PrinterState;
    wifi:WifiState = undefined;
    serverState:ServerState = undefined;
    screenTimer:any = undefined;
    screenTimeout:number = -1;
    screenHidden:boolean = false;
    screenEnergySavingInitalized:boolean = false;

    constructor(private server: RepetierServer,
                public modal: RSModal,
                public view: ViewContainerRef,
                public router: Router,
                private connectionHandler: ConnectionHandler,
                private serverAction: ServerAction,
                private redux:NgRedux<RepetierState>) {
        RSModal.defaultView = view;
        this.serverAction.initSetting("frontSetting", {
            "goWebInterface": false,
            "language": "en",
            "listPerPage": 10,
            "listSort": 0,
            "listType": 0,
            "showEmergency": true,
            "showFullscreen": false
        });
        this.serverAction.initSetting("screensaver",{timeout:10,energySaving:true});
        this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
            if (p.active) {
                let next = p.printers[p.active];
                let showStatus = false;
                if(this.printer && this.printer.slug == next.slug && next.printing && !this.printer.printing)
                    showStatus = true;
                this.printer = next;
                if (showStatus) {
                    this.router.navigateByUrl("/print-status");
                }
            } else this.printer = undefined;
        });
        this.serverSubscription = this.server$.subscribe((s:ServerState) => {
            this.serverState = s;
            if(s.settings && s.settings["screensaver"] && s.settings["screensaver"].timeout != this.screenTimeout) {
                this.screenTimeout = s.settings["screensaver"].timeout;
                this.initEnergySaving();
                this.resetScreensaver();
            }
        });
        this.wifiSubscription = this.wifi$.subscribe((w:WifiState) => {
            this.wifi = w;
            this.initEnergySaving();
        });
        window.document.body.addEventListener("mousemove",this.resetScreensaver);
        document.getElementById("screensaver").addEventListener("click",this.disableScreensaver);
    }

    initEnergySaving() {
        if(this.screenEnergySavingInitalized || !this.wifi || this.screenTimeout<0 || !this.wifi.screensaver)
            return;
        this.screenEnergySavingInitalized = true;
        if(this.screenTimeout == 0 || !this.serverState.settings["screensaver"].energySaving) {
            this.connectionHandler.send("screensaver",{enable:0,timeout:0});
        } else {
            this.connectionHandler.send("screensaver",{enable:1,timeout:this.screenTimeout*60+10});
        }
    }

    triggerScreensaver = () => {
        this.screenTimer = undefined;
        this.screenHidden = true;
        document.getElementById("screensaver").style.display = "block";
    };

    disableScreensaver = (event:MouseEvent) => {
        if(this.screenHidden) {
            this.screenHidden = false;
            document.getElementById("screensaver").style.display = "none";
            this.screenTimer = setTimeout(this.triggerScreensaver,this.screenTimeout*60000);
        }
        event.stopPropagation();
    };

    resetScreensaver = () => {
        if(this.screenHidden ) {
            return;
        }
        if(this.screenTimer)
            window.clearTimeout(this.screenTimer);
        if(this.screenTimeout <= 0)
            return; // disabled
        this.screenTimer = setTimeout(this.triggerScreensaver,this.screenTimeout*60000);
    };

    ngOnInit() {
        this.connectionHandler.connectFrontend();
        this.resetScreensaver();
    }

    ngOnDestroy() {
        document.getElementById("screensaver").removeEventListener("click",this.disableScreensaver);
        window.document.body.removeEventListener("mousemove",this.resetScreensaver);
        this.printersSubscription.unsubscribe();
        this.wifiSubscription.unsubscribe();
        this.serverSubscription.unsubscribe();
        this.connectionHandler.close();
        this.server.dispose();
    }
}