var core_1 = require('@angular/core');
var router_1 = require('@angular/router');
var RSFrontUI_1 = require("../RSFrontUI");
var RSBinding_1 = require("../RSBinding");
var GCodeInfo = (function () {
    function GCodeInfo(server, select, router) {
        this.server = server;
        this.select = select;
        this.router = router;
    }
    GCodeInfo.prototype.ngOnInit = function () {
        this.printer = this.server.printers.active;
        this.file = this.printer.activeGCode;
        this.file.updatePricing(this.printer);
    };
    GCodeInfo.prototype.ngDoCheck = function () {
        this.printer = this.server.printers.active;
        if (!this.printer)
            return;
    };
    GCodeInfo.prototype.selectFilament = function (idx) {
        var _this = this;
        this.select.select(this.server.settings.pricing.filaments, "n", "<?php _('Select Filament') ?>").then(function (f) {
            _this.file.perExtruder[idx].filament = f;
            _this.file.updatePricing(_this.printer);
        }, function () { });
    };
    GCodeInfo.prototype.print = function () {
        if (this.file.type == 1) {
            this.server.send("startJob", { id: this.file.id });
            this.router.navigateByUrl("/print-status");
        }
        else {
            var numWaiting = this.printer.queue.length;
            this.server.send("copyModel", { id: this.file.id });
            if (numWaiting == 0) {
                this.router.navigateByUrl("/print-status");
            }
            else {
                this.router.navigateByUrl("/print-queue");
            }
        }
    };
    GCodeInfo = __decorate([
        core_1.Component({
            selector: 'gcode-info',
            template: "\n<h1>{{file.name}}</h1>\n<div class=\"content-h1 margin-leftright15\" rs-drag>\n\n<div class=\"margin-bottom\">\n<rs-button class=\"primary\" (click)=\"print()\"><i class=\"rs rs-print\"></i> <?php _('Print') ?></rs-button>\n</div>\n\n<img [src]=\"file.imgM\">\n<h3><?php _('Analysis') ?></h3>\n<div class=\"value-list\">\n    <div><?php _('Printing Time') ?></div>\n    <div class=\"text-primary-bg\">{{file.printTime | dhms}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Total Price') ?></div>\n    <div class=\"text-primary-bg\">{{file.total | fixed:2}} {{file.currency}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Filament') ?></div>\n    <div class=\"text-primary-bg\">{{file.filamentTotal | fixed:1}} mm</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Volume') ?></div>\n    <div class=\"text-primary-bg\">{{(file.volumeTotal / 1000)| fixed:1}} cm<sup>3</sup></div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Weight') ?></div>\n    <div class=\"text-primary-bg\">{{file.weightTotal | fixed:1}} g</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Lines') ?></div>\n    <div class=\"text-primary-bg\">{{file.lines}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Layer') ?></div>\n    <div class=\"text-primary-bg\">{{file.layer}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Printed') ?></div>\n    <div class=\"text-primary-bg\">{{file.printed}} <?php _('times') ?></div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Uploaded') ?></div>\n    <div class=\"text-primary-bg\">{{file.created | date:'short'}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('File Length') ?></div>\n    <div class=\"text-primary-bg\">{{file.length | byte}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Slicer') ?></div>\n    <div class=\"text-primary-bg\">{{file.slicer}}</div>\n</div>\n<div class=\"value-list\">\n    <div><?php _('Volumetric Extrusion') ?></div>\n    <div class=\"text-primary-bg\">{{file.volumetric | yesno}}</div>\n</div>\n\n<h3 style=\"margin-top: 10px\"><?php _('Per Extruder Values') ?></h3>\n\n<div class=\"margin-bottom\" *ngFor=\"let e of file?.perExtruder; let i = index\">\n   <h4><?php _('Extruder') ?> {{e.extruder}}</h4>\n    <div class=\"value-list\">\n        <div><?php _('Filament') ?></div>\n        <div class=\"text-primary-bg\"><rs-button class=\"primary\" (click)=\"selectFilament(i)\">{{e.filament.n}}</rs-button></div>\n    </div>\n    <div class=\"value-list\">\n        <div><?php _('Length') ?></div>\n        <div class=\"text-primary-bg\">{{e.mm | fixed:0}}</div>\n    </div>\n    <div class=\"value-list\">\n        <div><?php _('Volume') ?></div>\n        <div class=\"text-primary-bg\">{{e.volume | fixed:1}} cm<sup>3</sup></div>\n    </div>\n    <div class=\"value-list\">\n        <div><?php _('Weight') ?></div>\n        <div class=\"text-primary-bg\">{{e.weight | fixed:1}} g</div>\n    </div>\n    <div class=\"value-list\">\n        <div><?php _('Material Costs') ?></div>\n        <div class=\"text-primary-bg\">{{e.price | fixed:2}} {{file.currency}}</div>\n    </div>\n</div>\n\n</div>\n    ",
            directives: [RSFrontUI_1.RSFRONTUI],
            pipes: [RSBinding_1.RSPIPES]
        }), 
        __metadata('design:paramtypes', [RSBinding_1.RepetierServer, RSFrontUI_1.RSSelect, router_1.Router])
    ], GCodeInfo);
    return GCodeInfo;
})();
exports.GCodeInfo = GCodeInfo;
var GCodesComponent = (function () {
    function GCodesComponent(modal, server, select) {
        this.modal = modal;
        this.server = server;
        this.select = select;
        this.page = 0;
    }
    GCodesComponent.prototype.ngDoCheck = function () {
        this.printer = this.server.printers.active;
        if (!this.printer)
            return;
        this.group = this.printer.activeGroup;
    };
    GCodesComponent.prototype.selectGroup = function () {
        var _this = this;
        this.select.select(this.printer.groups, "name", "<?php _('Select Group') ?>").then(function (grp) {
            _this.printer.activeGroup = grp;
        }, function () { });
    };
    GCodesComponent.prototype.selectGCode = function (gc) {
        this.printer.activeGCode = gc;
        this.page = 1;
    };
    GCodesComponent = __decorate([
        core_1.Component({
            selector: 'gcodes',
            template: "\n<rs-page>\n<side>\n    <rs-sideicon title=\"<?php _('Home') ?>\" icon=\"fa fa-home\" link=\"/\"></rs-sideicon>\n</side><side>\n    <!--<rs-sideicon title=\"Back\" icon=\"fa fa-reply\" link=\"/printer-list\"></rs-sideicon>-->\n</side>\n<side>\n    <rs-sideicon title=\"<?php _('Back') ?>\" icon=\"fa fa-reply\" (click)=\"page=0\" *ngIf=\"page>0\"></rs-sideicon>\n</side>\n<main>\n<div *ngIf=\"page==0\">\n<h1><?php _('G-Codes') ?></h1>\n<div class=\"content-h1\" rs-drag>\n    <rs-button class=\"secondary margin-leftright margin-bottom\" style=\"display:block\" (click)=\"selectGroup()\">{{group?.name}}</rs-button>\n    <div class=\"entry-list\">\n    <div class=\"entry left\" *ngFor=\"let f of printer?.models\" (click)=\"selectGCode(f)\">\n        <div class=\"left\">\n            <img [src]=\"f.imgT\">\n        </div>\n        <h3>{{f.name}}</h3>\n        <p>{{f.printTime|dhms}}</p>\n    </div>\n    </div>\n</div>\n</div>\n<gcode-info *ngIf=\"page==1\"></gcode-info>\n</main>\n</rs-page>\n    ",
            directives: [RSFrontUI_1.RSFRONTUI, GCodeInfo],
            pipes: [RSBinding_1.RSPIPES]
        }), 
        __metadata('design:paramtypes', [RSFrontUI_1.RSModal, RSBinding_1.RepetierServer, RSFrontUI_1.RSSelect])
    ], GCodesComponent);
    return GCodesComponent;
})();
exports.GCodesComponent = GCodesComponent;
//# sourceMappingURL=gcodes.component.js.map