/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Output, Input, EventEmitter, ChangeDetectionStrategy, ChangeDetectorRef} from '@angular/core';
import {Router} from '@angular/router';
import {RSModal, RSSelect} from "@repetier/front-ui";
import {
    RepetierServer,
    RepetierState,
    PrinterState,
    GroupState,
    GCodeFileState,
    PrinterAction,
    PrintersState,
    RSUtils,
    VisualAids
} from "@repetier/binding";
import {select, NgRedux} from 'ng2-redux';
import {Observable} from 'rxjs/Observable';
import {ISubscription} from "rxjs/Subscription";

@Component({
    selector: 'gcode-info',
    template: `
      <h1>{{file.name}}</h1>
      <div class="content-h1 margin-leftright15" rs-drag>

        <div class="margin-bottom">
          <rs-button *ngIf="!printer.printing && (file.type == 1 || printer.queue.length==0) && printer.online==1" [disabled]="printer.printing" [ngClass]="{primary: file.fits, error: !file.fits}" (hit)="print()">
            <i class="rs rs-print"></i> <?php _('Print') ?></rs-button>
          <rs-button *ngIf="file.type != 1 && (printer.queue.length>0 || printer.online!=1)" [ngClass]="{primary: file.fits, error: !file.fits}" (hit)="print()">
            <i class="rs rs-print"></i> <?php _('Queue File') ?></rs-button>
          <rs-button class="error" style="float:right" (hit)="deleteFile()">
            <i class="fa fa-trash-o"></i> <?php _('Delete') ?></rs-button>
          <div style="clear: both"></div>
        </div>

        <img [src]="printerAction.gcodeImageUrl(file.slug,file.list, file.id,'l')">
        <h3><?php _('Analysis') ?></h3>
        <div class="value-list">
          <div><?php _('Printing Time') ?></div>
          <div class="text-primary-bg">{{file.printTime | dhms}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Total Price') ?></div>
          <div class="text-primary-bg">{{file.total | fixed:2}} {{file.currency}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Filament') ?></div>
          <div class="text-primary-bg">{{file.filamentTotal | fixed:1}} mm</div>
        </div>
        <div class="value-list">
          <div><?php _('Volume') ?></div>
          <div class="text-primary-bg">{{(file.volumeTotal / 1000) | fixed:1}} cm<sup>3</sup></div>
        </div>
        <div class="value-list">
          <div><?php _('Weight') ?></div>
          <div class="text-primary-bg">{{file.weightTotal | fixed:1}} g</div>
        </div>
        <div class="value-list">
          <div><?php _('Lines') ?></div>
          <div class="text-primary-bg">{{file.lines}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Layer') ?></div>
          <div class="text-primary-bg">{{file.layer}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Printed') ?></div>
          <div class="text-primary-bg">{{file.printed}} <?php _('times') ?></div>
        </div>
        <div class="value-list">
          <div><?php _('Uploaded') ?></div>
          <div class="text-primary-bg">{{file.created | date:'short'}}</div>
        </div>
        <div class="value-list">
          <div><?php _('File Length') ?></div>
          <div class="text-primary-bg">{{file.length | byte}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Slicer') ?></div>
          <div class="text-primary-bg">{{file.slicer}}</div>
        </div>
        <div class="value-list">
          <div><?php _('Volumetric Extrusion') ?></div>
          <div class="text-primary-bg">{{file.volumetric | yesno}}</div>
        </div>

        <h3 style="margin-top: 10px"><?php _('Per Extruder Values') ?></h3>

        <div class="margin-bottom" *ngFor="let e of file?.perExtruder; let i = index">
          <h4><?php _('Extruder') ?> {{e.extruder}}</h4>
          <div class="value-list">
            <div><?php _('Filament') ?></div>
            <div class="text-primary-bg">
              <rs-button class="primary" (rsclick)="selectFilament(i)">{{e.filament.n}} <span class="caret"></span>
              </rs-button>
            </div>
          </div>
          <div class="value-list">
            <div><?php _('Length') ?></div>
            <div class="text-primary-bg">{{e.mm | fixed:0}}</div>
          </div>
          <div class="value-list">
            <div><?php _('Volume') ?></div>
            <div class="text-primary-bg">{{e.volume | fixed:1}} cm<sup>3</sup></div>
          </div>
          <div class="value-list">
            <div><?php _('Weight') ?></div>
            <div class="text-primary-bg">{{e.weight | fixed:1}} g</div>
          </div>
          <div class="value-list">
            <div><?php _('Material Costs') ?></div>
            <div class="text-primary-bg">{{e.price | fixed:2}} {{file.currency}}</div>
          </div>
        </div>

      </div>
    `,
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class GCodeInfo {
    printer: PrinterState;
    waitPrintStart: string = "";
    splitscreen: boolean = false
    @Input('file') file: GCodeFileState;
    @Output() close = new EventEmitter<boolean>();
    @select(['printers']) printers$: Observable<PrintersState>;

    private printersSubscription: ISubscription;

    constructor(public server: RepetierServer,
                public select: RSSelect,
                public router: Router,
                private modal: RSModal,
                private printerAction: PrinterAction,
                private visualAid: VisualAids,
                private redux: NgRedux<RepetierState>,
                private ref: ChangeDetectorRef) {
        this.printersSubscription = this.printers$.subscribe(printers => {
            if (this.printer != printers.printers[printers.active]) {
                this.printer = printers.printers[printers.active];
                if (this.waitPrintStart.length > 0 && this.printer.jobid > 0 && this.printer.printing == true) {
                    this.router.navigateByUrl("/print-status");
                }
                ref.markForCheck();
            }
        });
        window.addEventListener("resize", this.updateSize);
    }

    ngOnDestroy() {
        window.removeEventListener("resize", this.updateSize);
        this.printersSubscription.unsubscribe();
        if (this.waitPrintStart.length > 0)
            this.visualAid.hideBusy(this.waitPrintStart);
    }

    updateSize = (): void => {
        this.splitscreen = document.body.clientWidth >= 800
    };

    selectFilament(idx: number) {
        this.select.select(this.redux.getState().server.settings["pricing"].filaments, "n", "<?php _('Select Filament') ?>").then((f) => {
            let newFile = RSUtils.copyObject(this.file);
            newFile.perExtruder[idx].filament = f;
            newFile = this.printerAction.gcodeUpdatePricing(newFile);
            this.printerAction.updateGCode(newFile);
        }, () => {
        });
    }

    print() {
        if (this.file.fits) {
            this.printNoQuestion()
        } else {
            this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('According to your defined printer limits, this print will not fit inside your printer. Print anyway?') ?>").then(() => {
                this.printNoQuestion()
            }, () => {
            });

        }
    }

    printNoQuestion() {

        if (this.file.type == 1) {
            if (this.printer.online == 1 && !this.printer.printing) {
                this.waitPrintStart = this.visualAid.showBusy("<?php _('Preparing print ...') ?>");
                this.printerAction.gcodePrint(this.file);
            }
        } else {
            let numWaiting = this.printer.queue.length;
            if (numWaiting == 0 && this.printer.online == 1 && !this.printer.printing) {
                this.waitPrintStart = this.visualAid.showBusy("<?php _('Preparing print ...') ?>");
                this.printerAction.gcodePrint(this.file);
            } else {
                this.printerAction.gcodePrint(this.file);
                this.router.navigateByUrl("/print-queue");
            }
        }
    }

    deleteFile() {
        this.modal.confirm("<?php _('Confirmation required') ?>", "<?php _('Do you really want to delete $1?') ?>".replace('$1', this.file.name)).then(() => {
            this.printerAction.gcodeDelete(this.file);
            this.close.emit(true);
        }, () => {
        });
    }
}

@Component({
    selector: 'gcodes',
    template: `
      <rs-page>
        <side>
          <rs-sideicon title="<?php _('Home') ?>" icon="fa fa-home" link="/"></rs-sideicon>
        </side>
        <side>
          <!--<rs-sideicon title="Back" icon="fa fa-reply" link="/printer-list"></rs-sideicon>-->
        </side>
        <side>
          <rs-sideicon title="<?php _('Back') ?>" icon="fa fa-reply" (rsclick)="page=0" *ngIf="page>0 && !splitscreen"></rs-sideicon>
        </side>
        <main>
          <div *ngIf="page==0 && !splitscreen">
            <h1><?php _('G-Codes') ?></h1>
            <div class="content-h1" rs-drag>
              <div class="margin-leftright margin-bottom">
                <rs-button class="secondary full" style="display:block" (hit)="selectGroup()">
                  {{group?.name}} <span class="caret"></span></rs-button>
              </div>
              <div class="entry-list">
                <div class="entry left pointer" *ngFor="let f of printer?.modelsFiltered" (rsclick)="selectGCode(f)">
                  <div class="left">
                    <img [src]="printerAction.gcodeImageUrl(f.slug,f.list, f.id,'s')">
                  </div>
                  <h3>{{f.name}}</h3>
                  <p>{{f.printTime | dhms}}</p>
                </div>
              </div>
            </div>
          </div>
          <div *ngIf="splitscreen" style="display: flex;">
            <div style="flex: 1;">
            <h1><?php _('G-Codes') ?></h1>
            <div class="content-h1" rs-drag>
              <div class="margin-leftright margin-bottom">
              <rs-button class="secondary full" style="display:block" (hit)="selectGroup()">
                {{group?.name}} <span class="caret"></span></rs-button>
              </div>
              <div class="entry-list">
                <div class="entry left pointer" *ngFor="let f of printer?.modelsFiltered" (rsclick)="selectGCode(f)">
                  <div class="left">
                    <img [src]="printerAction.gcodeImageUrl(f.slug,f.list, f.id,'s')">
                  </div>
                  <h3>{{f.name}}</h3>
                  <p>{{f.printTime | dhms}}</p>
                </div>
              </div>
            </div>
            </div>
            <div style="flex: 1;">
              <gcode-info *ngIf="activeGCode" [file]="activeGCode"></gcode-info>
            </div>
          </div>
          <gcode-info *ngIf="page==1 && !splitscreen" (close)="page=0" [file]="activeGCode"></gcode-info>
        </main>
      </rs-page>
    `,
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class GCodesComponent {
    group: GroupState = undefined;
    printer: PrinterState = undefined;
    activeGCode: GCodeFileState = undefined;
    page: number = 0;
    splitscreen: boolean = false
    @select(['printers']) printers$: Observable<PrintersState>;

    private printersSubscription: ISubscription;

    constructor(public modal: RSModal,
                public server: RepetierServer,
                public select: RSSelect,
                private printerAction: PrinterAction,
                private ref: ChangeDetectorRef) {
        this.printersSubscription = this.printers$.subscribe(printers => {
            if (!printers.active) return; // on load it can be undefined!
            if (this.printer != printers.printers[printers.active]) {
                this.printer = printers.printers[printers.active];
                if (this.printer.groupsSorted && this.printer.groupsSorted.length > 0) {
                    this.group = this.printer.groups[this.printer.activeGroupId];
                    this.activeGCode = this.printerAction.gcodeGetLatestVersion(this.activeGCode);
                }
                ref.markForCheck();
            }
        });
        window.addEventListener("resize", this.updateSize);
        this.updateSize();
    }

    ngOnDestroy() {
        window.removeEventListener("resize", this.updateSize);
        this.printersSubscription.unsubscribe();
    }

    updateSize = (): void => {
        this.splitscreen = document.body.clientWidth >= 800
    };

    selectGroup() {
        this.select.select(this.printer.groupsSorted, "name", "<?php _('Select Group') ?>").then((grp) => {
            this.printerAction.setActiveGroupId(this.printer.slug, grp.id);
        }, () => {
        });
    }

    selectGCode(gc: any) {
        this.activeGCode = gc;
        this.page = 1;
    }
}

