/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component} from '@angular/core';
import {PrinterState, PrintersState, PrinterAction} from "@repetier/binding";
import {select} from 'ng2-redux';
import {Observable} from 'rxjs/Observable';
import {ISubscription} from "rxjs/Subscription";
import {RSUIHelper} from "@repetier/front-ui";
import {Router} from "@angular/router";

@Component({
    selector: 'rs-print-status',
    template: `
      <rs-page>
        <side>
          <rs-sideicon title="<?php _('Home') ?>" icon="fa fa-home" link="/"></rs-sideicon>
        </side>
        <side>
          <rs-sideicon *ifConnected="true" title="<?php _('Temp.') ?>" icon="rs rs-chart" link="/temperatures"></rs-sideicon>
        </side>
        <side>
          <rs-sideicon *ifConnected="true" title="<?php _('Params') ?>" icon="rs rs-params" link="/params"></rs-sideicon>
        </side>
        <main>
          <div *ifPrinting="true">
            <h1>
              <span *ngIf="printer.paused"><?php _('Paused:') ?></span><span *ngIf="!printer.paused"><?php _('Printing:') ?></span>
              {{printer?.printJob}}</h1>
            <div class="content-h1 margin-leftright15">
              <div class="centered-grid stacked content-h1">
                <div class="regular-grid margin-leftright" style="width: 100%;">
                  <rs-gauge [width]="height1" [height]="height1" [value]="printer.printDone" [rangeWidth]="rangeWidth" [valueWidth]="rangeWidth" divider="true" type="primary">
                    <span class="text-primary-bg" style="font-size:0.6rem"><?php _('ETE') ?></span><br>
                    {{printer.printedTime - printer.printedTimeComp | hmsShort}}
                    <div style="height:5px"></div>
                    {{((printers$ | async).stateTime + 1000 * (printer.printedTime - printer.printedTimeComp)) | toint |
                  date:'shortTime'}}<br>
                    <span class="text-primary-bg" style="font-size:0.6rem"><?php _('ETA') ?></span>
                  </rs-gauge>
                  <img [src]="printerAction.printingImage(printer.slug,'m')" style="border:1px solid #000;" [style.height]="height1+'px'">
                </div>
                <div class="regular-grid margin-leftright" style="width: 100%;">
                  <div>
                    <div class="value-list">
                      <div><i class="fa fa-arrow-up"></i> <span class="hide-max-320"><?php _('Z Position') ?></span>
                      </div>
                      <div class="text-primary-bg">{{printer?.posZ | fixed:2}} mm</div>
                    </div>
                    <div class="value-list">
                      <div><i class="fa fa-bars"></i> <span class="hide-max-320"><?php _('Layer') ?></span></div>
                      <div class="text-primary-bg">{{printer?.layer}} / {{printer?.ofLayer}}</div>
                    </div>
                    <div class="value-list" *ngIf="printer?.config?.extruders?.length>0">
                      <div><i class="rs rs-extruder"></i>
                        <span class="hide-max-320">{{printerAction.extruderName(printer.slug, printer.activeExtruderId)}}</span>&nbsp;
                      </div>
                      <div class="text-primary-bg">{{printer?.extruders[printer.activeExtruderId]?.currentTemp | fixed:0}}
                        /
                        {{printer?.extruders[printer.activeExtruderId]?.tempSet | fixed:0}}°C
                      </div>
                    </div>
                    <div class="value-list" *ngIf="printer?.config?.heatedBed?.installed">
                      <div><i class="rs rs-heated-bed"></i> <span class="hide-max-320"><?php _('Bed') ?></span></div>
                      <div class="text-primary-bg">{{printer?.heatedBed?.currentTemp | fixed:0}} /
                        {{printer?.heatedBed?.tempSet | fixed:0}}°C
                      </div>
                    </div>
                    <div class="value-list" *ngIf="printer?.config?.heatedChamber?.installed">
                      <div><i class="rs rs-chamber"></i> <span class="hide-max-320"><?php _('Chamber') ?></span></div>
                      <div class="text-primary-bg">{{printer?.heatedChamber?.currentTemp | fixed:0}} /
                        {{printer?.heatedChamber?.tempSet | fixed:0}}°C
                      </div>
                    </div>
                    <div class="value-list" *ngIf="showExtraParams && printer?.config?.general?.fan">
                      <div><i class="rs rs-fan"></i> <span class="hide-max-320"><?php _('Fan') ?></span></div>
                      <div class="text-primary-bg">{{printer.fanPercent | fixed:0}}%</div>
                    </div>
                    <div class="value-list" *ngIf="showExtraParams">
                      <div><i class="rs rs-speed"></i> <span class="hide-max-320"><?php _('Speed') ?></span></div>
                      <div class="text-primary-bg">{{printer.speedMultiply | fixed:0}}%</div>
                    </div>
                    <div class="value-list" *ngIf="showExtraParams">
                      <div><i class="rs rs-flow"></i> <span class="hide-max-320"><?php _('Flow') ?></span></div>
                      <div class="text-primary-bg">{{printer.flowMultiply | fixed:0}}%</div>
                    </div>
                  </div>
                  <div>
                    <div class="regular-grid stacked" style="height:80px">
                      <div style="width: 120px;">
                        <rs-button class="primary full" (hit)="printerAction.pausePrint(printer.slug)" *ngIf="!printer?.paused">
                          <i class="fa fa-pause"></i> <?php _('Pause') ?></rs-button>
                        <rs-button class="primary full" (hit)="printerAction.unpausePrint(printer.slug)" [disabled]="printer.pauseState==3" *ngIf="printer?.paused">
                          <i class="fa fa-play"></i> {{pauseNames[printer.pauseState]}}
                        </rs-button>
                      </div>
                      <div style="width: 120px;">
                        <rs-button class="error full" (hit)="ui.stopPrint()">
                          <i class="fa fa-stop"></i>  <?php _('Stop') ?></rs-button>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
          <div *ifPrinting="false">
            <h1><?php _('Idle') ?></h1>
            <div class="content-h1 margin-leftright15">
              <div class="alert info margin">
                <?php _('Printer is idle') ?>
              </div>
            </div>
          </div>
        </main>
      </rs-page>`
})
export class PrintStatusComponent {
    printer: PrinterState;
    active: PrinterState = undefined;
    @select('printers') printers$: Observable<PrintersState>;
    private printersSubscription: ISubscription;
    height1: number = 100;
    rangeWidth: number = 12;
    showExtraParams: boolean = false;
    pauseNames: any = ["<?php _('Pause') ?>", "<?php _('Continue') ?>", "<?php _('Preheat') ?>", "<?php _('Heating...') ?>", "<?php _('Continue') ?>"]

    constructor(public ui: RSUIHelper,
                public router: Router,
                private printerAction: PrinterAction) {
        this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
            if (p.active) {
                this.printer = p.printers[p.active];
                if (this.printer.printing == false) {
                    this.router.navigateByUrl("/");
                }
            } else this.printer = undefined;
        });
        this.updateSize();
        window.addEventListener("resize", this.updateSize);
    }

    updateSize = (): void => {
        if (document.body.clientHeight > 300)
            this.height1 = 0.45 * document.body.clientHeight;
        else
            this.height1 = 100;
        this.showExtraParams = document.body.clientHeight > 400;
        this.rangeWidth = document.body.clientHeight / 20;
    };

    ngOnDestroy(): void {
        window.removeEventListener("resize", this.updateSize);
        this.printersSubscription.unsubscribe();
    }
}
