/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component} from '@angular/core';
import {RSIntKeyboard} from "@repetier/front-ui";
import {ServerState, ServerAction,WifiState,ConnectionHandler} from "@repetier/binding";
import {select} from 'ng2-redux';
import {Observable} from 'rxjs/Observable';
import {ISubscription} from "rxjs/Subscription";

@Component({
    selector: 'rs-params-component',
    template: `
<rs-page>
<side>
    <rs-sideicon title="<?php _('Home') ?>" icon="fa fa-home" link="/"></rs-sideicon>
</side>
<side>
</side>
<side></side>
<main>
    <h1><?php _('Screensaver') ?></h1>
    <div class="content-h1 " style="padding: 0;" rs-drag>
       <div class="iconlist">
           <div class="button-icon sep-icon">
              <rs-button class="secondary outline full" (hit)="setTime()">
              <i class="rs rs-screensaver fw"></i> <?php _('Screensaver Timeout') ?><br>
              {{server?.settings["screensaver"].timeout}} <?php _('minutes') ?></rs-button>
           </div>
       </div>
        <div ng-if="wifi?.screensaver && server" class="margin-leftright">
            <rs-switch [checked]="energySend" (checkedChange)="toggleEnergy($event)"></rs-switch> <?php _('Disable Display') ?>
        </div>
       <div class="alert info margin-top margin-leftright">
           <?php _('If your screen goes black after some time, set timout to that time to protect for unintentionally activating functions. If not, use it to dim screen or set to 0 to disable it.') ?>
       </div>
    </div>
</main>
</rs-page>
    `
})

export class ScreensaverComponent {
    @select('server') server$: Observable<ServerState>;
    @select('wifi') wifi$: Observable<WifiState>;
    server: ServerState = undefined;
    wifi:WifiState = undefined;

    private serverSubscription: ISubscription;
    private wifiSubscription: ISubscription;
    energySend:boolean = undefined;

    constructor(private intKbd: RSIntKeyboard,
                private serverAction:ServerAction,
                private  con:ConnectionHandler) {
        this.serverSubscription = this.server$.subscribe((p: ServerState) => {
            if(p.settings && p.settings["screensaver"]) {
                this.server = p;
                if (this.energySend === undefined)
                    this.energySend = p.settings["screensaver"].energySaving;
            }
        });
        this.wifiSubscription = this.wifi$.subscribe((p: WifiState) => {
                this.wifi = p;
        });
    }

    ngOnDestroy(): void {
        this.serverSubscription.unsubscribe();
        this.wifiSubscription.unsubscribe();
    }

    toggleEnergy(newVal):void {
        if(this.energySend === undefined || this.server === undefined || !this.server.settings["screensaver"])
            return; // break loop
        this.energySend = newVal;
        this.serverAction.updateSetting("screensaver",Object.assign({},this.server.settings["screensaver"],{energySaving:newVal}));
        if(this.server.settings["screensaver"].timeout == 0 || !newVal) {
            this.con.send("screensaver",{enable:0,timeout:0});
        } else {
            this.con.send("screensaver",{enable:1,timeout:this.server.settings["screensaver"].timeout*60+10});
        }
    }
    setTime(): void {
        this.intKbd.getInt(this.server.settings["screensaver"].timeout, '<i class="rs rs-screensaver"></i> ' + "<?php _('Screensaver') ?>", {
            min: 0,
            max: 240,
            unit: "<?php _('minutes') ?>",
            presets: [0, 5, 10, 15]
        }).then(
            (v) => {
                this.serverAction.updateSetting("screensaver",Object.assign({},this.server.settings["screensaver"],{timeout:v}));
                if(v > 0 && this.energySend)
                    this.con.send("screensaver",{enable:1,timeout:v*60+10}); // modify
            }, () => {
            }
        );
    }
}