/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var Byte = (function () {
    function Byte() {
    }
    Byte.prototype.transform = function (input) {
        if (input < 1024)
            return input + " <?php _('byte') ?>";
        input /= 1024.0;
        if (input < 1024)
            return input.toFixed(1) + " <?php _('kB') ?>";
        input /= 1024.0;
        if (input < 1024)
            return input.toFixed(1) + " <?php _('MB') ?>";
        input /= 1024.0;
        return input.toFixed(1) + " <?php _('GB') ?>";
    };
    Byte = __decorate([
        core_1.Pipe({ name: 'byte' }), 
        __metadata('design:paramtypes', [])
    ], Byte);
    return Byte;
})();
exports.Byte = Byte;
var YesNo = (function () {
    function YesNo() {
    }
    YesNo.prototype.transform = function (input) {
        if (input)
            return "<?php _('Yes') ?>";
        return "<?php _('No') ?>";
    };
    YesNo = __decorate([
        core_1.Pipe({ name: 'yesno' }), 
        __metadata('design:paramtypes', [])
    ], YesNo);
    return YesNo;
})();
exports.YesNo = YesNo;
var Coord = (function () {
    function Coord() {
    }
    Coord.prototype.transform = function (input, precision) {
        if (input === null || input === undefined) {
            return "------." + "--------".substr(0, precision);
        }
        var r = input.toFixed(precision);
        var extra = 6 + precision - r.length;
        while (extra > 0) {
            extra--;
            r = "&nbsp;" + r;
        }
        return r;
    };
    Coord = __decorate([
        core_1.Pipe({ name: 'coord' }), 
        __metadata('design:paramtypes', [])
    ], Coord);
    return Coord;
})();
exports.Coord = Coord;
var Fixed = (function () {
    function Fixed() {
    }
    Fixed.prototype.transform = function (input, precision) {
        if (input === null || input === undefined) {
            return "-." + "--------".substr(0, precision);
        }
        return input.toFixed(precision);
    };
    Fixed = __decorate([
        core_1.Pipe({ name: 'fixed' }), 
        __metadata('design:paramtypes', [])
    ], Fixed);
    return Fixed;
})();
exports.Fixed = Fixed;
function twoDigits(a) {
    a = a.toFixed(0);
    if (a.length == 1)
        return "0" + a;
    return a;
}
var HMS = (function () {
    function HMS() {
    }
    HMS.prototype.transform = function (input) {
        if (input === undefined)
            return "";
        if (input < 0)
            return "<?php _('Computing ...') ?>";
        var hours = Math.floor(input / 3600);
        input -= 3600 * hours;
        var min = Math.floor(input / 60);
        input -= min * 60;
        var t = "";
        if (hours > 0)
            t += hours + "h ";
        if (min > 0 || hours > 0)
            t += twoDigits(min) + "m ";
        return t + twoDigits(input) + "s";
    };
    HMS = __decorate([
        core_1.Pipe({ name: 'hms' }), 
        __metadata('design:paramtypes', [])
    ], HMS);
    return HMS;
})();
exports.HMS = HMS;
var HMSShort = (function () {
    function HMSShort() {
    }
    HMSShort.prototype.transform = function (input) {
        if (input === undefined)
            return "";
        if (input < 0)
            return "<?php _('Computing ...') ?>";
        var hours = Math.floor(input / 3600);
        input -= 3600 * hours;
        var min = Math.floor(input / 60);
        input -= min * 60;
        var t = "";
        if (hours > 0)
            t += hours + "h ";
        if (min > 0 || hours > 0)
            t += twoDigits(min) + "m";
        if (hours > 0)
            return t;
        return t + " " + twoDigits(input) + "s";
    };
    HMSShort = __decorate([
        core_1.Pipe({ name: 'hmsShort' }), 
        __metadata('design:paramtypes', [])
    ], HMSShort);
    return HMSShort;
})();
exports.HMSShort = HMSShort;
var DHMS = (function () {
    function DHMS() {
    }
    DHMS.prototype.transform = function (input) {
        if (input === undefined)
            return "";
        if (input < 0)
            return "<?php _('Computing ...') ?>";
        var hours = Math.floor(input / 3600);
        input -= 3600 * hours;
        var days = 0;
        if (hours > 23) {
            days = Math.floor(hours / 24);
            hours -= days * 24;
        }
        var min = Math.floor(input / 60);
        input -= min * 60;
        var t = "";
        if (days > 0)
            t += days + "d ";
        if (hours > 0 || days > 0)
            t += hours + "h ";
        if (min > 0 || hours > 0)
            t += twoDigits(min) + "m ";
        return t + twoDigits(input) + "s";
    };
    DHMS = __decorate([
        core_1.Pipe({ name: 'dhms' }), 
        __metadata('design:paramtypes', [])
    ], DHMS);
    return DHMS;
})();
exports.DHMS = DHMS;
var EqualsPipe = (function () {
    function EqualsPipe() {
    }
    EqualsPipe.prototype.transform = function (value, name, filt) {
        if (value === undefined)
            return value;
        return value.filter(function (item) { return item[name] == filt; });
    };
    EqualsPipe = __decorate([
        core_1.Pipe({ name: 'equals' }), 
        __metadata('design:paramtypes', [])
    ], EqualsPipe);
    return EqualsPipe;
})();
exports.EqualsPipe = EqualsPipe;
exports.RSPIPES = [Byte, Coord, Fixed, HMS, HMSShort, DHMS, YesNo, EqualsPipe];
//# sourceMappingURL=Pipes.js.map