/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var RSUtils = (function () {
    function RSUtils() {
    }
    RSUtils.extend = function (obj, extendBy) {
        for (var i in extendBy) {
            if (extendBy.hasOwnProperty(i)) {
                obj[i] = extendBy[i];
            }
        }
        return obj;
    };
    RSUtils.copyObject = function (extendBy) {
        var obj = {};
        for (var i in extendBy) {
            if (extendBy.hasOwnProperty(i)) {
                var tp = Object.prototype.toString.call(extendBy[i]);
                if (tp === "[object Array]") {
                    obj[i] = this.copyArray(extendBy[i]);
                }
                else if (tp === "[object Object]") {
                    obj[i] = this.copyObject(extendBy[i]);
                }
                else if (tp !== "[object Function]") {
                    obj[i] = extendBy[i];
                }
            }
        }
        return obj;
    };
    RSUtils.copyArray = function (arr) {
        var a = [];
        for (var i in arr) {
            var tp = Object.prototype.toString.call(arr[i]);
            if (tp === "[object Array]") {
                a.push(this.copyArray(arr[i]));
            }
            else if (tp === "[object Object]") {
                a.push(this.copyObject(arr[i]));
            }
            else if (tp !== "[object Function]") {
                a.push(arr[i]);
            }
        }
    };
    RSUtils.createAlphaID = function (chars) {
        var id = "";
        var allowedChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        for (var i = 0; i < chars; i++)
            id += allowedChars.charAt(Math.floor(Math.random() * allowedChars.length));
        return id;
    };
    RSUtils.equals = function (a, b) {
        var at = Object.prototype.toString.call(a);
        var bt = Object.prototype.toString.call(b);
        if (at != bt)
            return false;
        if (at === "[object Array]") {
            if (a.length != b.length)
                return false;
            for (var idx in a) {
                if (!RSUtils.equals(a[idx], b[idx]))
                    return false;
            }
        }
        else if (at === "[object Object]") {
            for (var idx in a) {
                if (a.hasOwnProperty(idx)) {
                    if (!RSUtils.equals(a[idx], b[idx]))
                        return false;
                }
            }
            for (var idx in b) {
                if (b.hasOwnProperty(idx)) {
                    if (!RSUtils.equals(a[idx], b[idx]))
                        return false;
                }
            }
        }
        else if (at !== "[object Function]") {
            return a === b;
        }
        return true;
    };
    RSUtils.isEmptyObject = function (obj) {
        for (var idx in obj) {
            if (obj.hasOwnProperty(idx))
                return false;
        }
        return true;
    };
    RSUtils.sortOnField = function (arr, key, desc) {
        if (desc == undefined)
            desc = false;
        if (desc) {
            return arr.sort(function (a, b) {
                if (a[key] > b[key]) {
                    return -1;
                }
                else if (a[key] < b[key]) {
                    return 1;
                }
                return 0;
            });
        }
        return arr.sort(function (a, b) {
            if (a[key] < b[key]) {
                return -1;
            }
            else if (a[key] > b[key]) {
                return 1;
            }
            return 0;
        });
    };
    return RSUtils;
})();
exports.RSUtils = RSUtils;
var Deferred = (function () {
    function Deferred() {
        this.resolve = null;
        this.reject = null;
        this.promise = null;
        this.promise = new Promise(function (resolve, reject) {
            this.resolve = resolve ? resolve : function () {
            };
            this.reject = reject ? reject : function () {
            };
            if (!reject)
                console.log("no reject fount");
        }.bind(this));
    }
    return Deferred;
})();
exports.Deferred = Deferred;
var SerializationHelper = (function () {
    function SerializationHelper() {
    }
    SerializationHelper.toInstance = function (obj, json) {
        var jsonObj = JSON.parse(json);
        if (typeof obj["fromJSON"] === "function") {
            obj["fromJSON"](jsonObj);
        }
        else {
            for (var propName in jsonObj) {
                obj[propName] = jsonObj[propName];
            }
        }
        return obj;
    };
    SerializationHelper.convert = function (obj, jsonObj) {
        if (typeof obj["fromJSON"] === "function") {
            obj["fromJSON"](jsonObj);
        }
        else {
            for (var propName in jsonObj) {
                obj[propName] = jsonObj[propName];
            }
        }
        return obj;
    };
    return SerializationHelper;
})();
exports.SerializationHelper = SerializationHelper;
//# sourceMappingURL=RSUtils.js.map