/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var Subject_1 = require("rxjs/Subject");
var Printers_1 = require("./Printers");
var User_1 = require("./User");
var RSUtils_1 = require("./RSUtils");
var RSUtils_2 = require("./RSUtils");
var Wifi_1 = require("./Wifi");
// This class should show a hint about connection status, when the connection is lost.
// Derive from it for a implementation of your liking.
var VisualAids = (function () {
    function VisualAids() {
    }
    VisualAids.prototype.onConnected = function () {
        //console.log("Connected");
    };
    VisualAids.prototype.onDisconnected = function () {
        //console.log("Disconnected");
    };
    VisualAids.prototype.onPrinterError = function () { };
    VisualAids.prototype.onPrinterErrorSolved = function () { };
    VisualAids.prototype.showBusy = function (reason) { };
    VisualAids.prototype.hideBusy = function () { };
    return VisualAids;
})();
exports.VisualAids = VisualAids;
var Language = (function () {
    function Language() {
    }
    return Language;
})();
exports.Language = Language;
var ExternalCommand = (function () {
    function ExternalCommand(v) {
        this.id = v.id;
        this.name = v.id;
        this.confirm = v.confirm;
        this.local = v.local;
        this.remote = v.remote;
        this.permPrint = v.permPrint;
        this.permConfig = v.permConfig;
        this.permAdd = v.permAdd;
        this.permDel = v.permDel;
    }
    return ExternalCommand;
})();
exports.ExternalCommand = ExternalCommand;
var ServerEvent = (function () {
    function ServerEvent(event) {
        this.event = event;
    }
    return ServerEvent;
})();
exports.ServerEvent = ServerEvent;
var ServerFolder = (function () {
    function ServerFolder() {
    }
    return ServerFolder;
})();
exports.ServerFolder = ServerFolder;
var RepetierServer = (function () {
    function RepetierServer(app, http) {
        var _this = this;
        this.app = app;
        this.http = http;
        this.disposed = false;
        this.disconnectVisual = new VisualAids();
        this.server = window.location.host;
        this.session = undefined;
        this.apikey = undefined;
        this.authenticated = false;
        this.globals = {}; // universal execution permanent storage for data to survive page switches.
        this.permissions = 0;
        this.connected = false;
        this.canPrint = false;
        this.canDelete = false;
        this.canUpload = false;
        this.canConfigure = false;
        this.frontend = false;
        this.language = "en";
        this.httpType = "http";
        this.settings = {
            pricing: {
                currency: 'EUR',
                perPrint: 5.0,
                perHour: 0.3,
                filaments: [{ u: RSUtils_1.RSUtils.createAlphaID(10), n: 'PLA', p: 25.0, w: 1.25 }],
                map: {}
            }
        }; // server settings
        this.conDialogVisible = false;
        // Keep all pending requests here until they get responses
        this.callbacks = {};
        // Create a unique callback ID to map requests to responses
        this.currentCallbackId = 0;
        // Create our websocket object with the address to the websocket
        this.wasConnected = false;
        this.alert = false;
        this.deflist = [];
        this.printerSlug = '';
        this.conDialog = null;
        this.openPings = 0;
        this.firstConnection = true;
        this.nextTimeout = 50;
        this.startVersion = "";
        this.ignoreErrorsTil = -1;
        this.update = {};
        this.folders = [];
        this.printerErrors = {};
        this.languages = [];
        this.localEventListener = function (evt) {
            switch (evt.event) {
                case "connected":
                    // Read external commands
                    _this.send("listExternalCommands").then(function (data) {
                        _this.allExternalCommands = data;
                        _this.updateExtCommands();
                    });
                    _this.wifi.updateRouterList();
                    break;
                case "disconnected":
                    break;
            }
        };
        this.eventListener = function (evt) {
            var data = evt.data;
            switch (evt.event) {
                case "settingChanged":
                    RSUtils_1.RSUtils.extend(_this.settings, data);
                    _this._originalSettings = RSUtils_1.RSUtils.copyObject(_this.settings);
                    break;
                case "foldersChanged":
                    _this.updateFolders();
                    break;
                case "wifiChanged":
                    _this.wifi.setWifiList(data);
                    break;
            }
        };
        // Do a reconnect or first connect
        this.startConnection = function () {
            var con = _this;
            if (_this.ws != null && _this.ws.readyState == 1)
                return;
            var auth = undefined; //localStorageService.get("session");
            var extra = "";
            _this.ignoreErrorsTil = -1; // reset error handling and callbacks
            if (_this.frontend) {
                extra += "&front=1";
            }
            if (_this.apikey) {
                extra += "&apikey=" + encodeURIComponent((_this.apikey));
            }
            else if (auth && auth.session) {
                extra += "&sess=" + encodeURIComponent(auth.session);
            }
            //log("opening socket");
            _this.ws = new WebSocket((RepetierServer.isSecure() ? "wss://" : "ws://") + _this.server + "/socket/?lang=" + _this.language + extra);
            _this.ws.onopen = function () {
                con.connected = true;
                //console.log("Connected");console.log(con);
                con.openPings = 0;
                if (con.conDialogVisible) {
                    con.disconnectVisual.onConnected();
                }
                con.conDialogVisible = false;
                //con.updateImportant();
                var oldCallbacks = con.callbacks;
                con.callbacks = [];
                var tryFlush = function () {
                    if (con.ws.readyState != 1) {
                        setTimeout(tryFlush, 20);
                        return;
                    }
                    con.firstConnection = false;
                    con.updateImportant();
                    for (var _i = 0, _a = con.deflist; _i < _a.length; _i++) {
                        var val = _a[_i];
                        val.resolve(con.ws);
                    }
                    con.deflist = [];
                    con.nextTimeout = 50;
                    con.wasConnected = true;
                    con.fireLocalEvent("connected");
                };
                // all commands are unanswered and need to be resend once connection is established
                for (var cbidx in oldCallbacks) {
                    var cb = oldCallbacks[cbidx];
                    (function (cb) {
                        var d = new RSUtils_1.Deferred();
                        con.deflist.push(d);
                        d.promise.then(function () {
                            //cb.request.session = User.getSession();
                            if (!cb.request.ws || cb.request.ws !== con.ws) {
                                delete (cb.request.ws);
                                //console.log("fulfill ");
                                //console.log(con.ws);
                                con.ws.send(JSON.stringify(cb.request));
                                cb.request.ws = con.ws;
                            }
                        });
                    })(cb);
                }
                setTimeout(tryFlush, 10);
            };
            _this.ws.onerror = function (message) {
                //console.log("websocket error");
                //console.log(message);
            };
            _this.ws.onclose = function (message) {
                console.log("Disconnected " + message.code);
                if (con.connected) {
                    con.connected = false;
                    con.fireLocalEvent("disconnected");
                    if (!con.firstConnection) {
                        con.conDialogVisible = true;
                        con.disconnectVisual.onDisconnected();
                    }
                }
                setTimeout(function () { return con.startConnection(); }, con.nextTimeout);
                if (con.nextTimeout < 1000)
                    con.nextTimeout = 2 * con.nextTimeout;
                else
                    con.nextTimeout = 2000;
            };
            _this.ws.onmessage = function (message) {
                try {
                    //console.log("msg");console.log(message);
                    con.listener(JSON.parse(message.data));
                }
                catch (error) {
                    console.log("websocket listener");
                    console.log(error);
                    console.log(message);
                }
            };
        };
        this.hasFeature1 = false;
        this.ping = function () {
            //console.log("ping "+this.connected);
            if (_this.connected) {
                if (_this.openPings > 10)
                    _this.ws.close();
                else {
                    _this.openPings++;
                    _this.send("ping").then(function () {
                        _this.openPings--;
                    }); // we are still alive!
                }
            }
            if (!_this.disposed)
                setTimeout(function () { return _this.ping(); }, 2000);
        };
        this._originalSettings = {};
        //console.log("creating server instance");
        this.serverEvents = new Subject_1.Subject();
        this.localEvents = new Subject_1.Subject();
        if (RepetierServer.isSecure())
            this.httpType = "https";
        this.printers = new Printers_1.Printers(this);
        this.wifi = new Wifi_1.Wifi(this);
        window.rs = this;
        this.http.get("/languages.json").
            map(function (res) { return res.json(); }).
            subscribe(function (data) {
            _this.languages = data;
        });
        this.user = new User_1.User(this);
        this.eventSubscription = this.serverEvents.subscribe(this.eventListener);
        this.localEventSubscription = this.localEvents.subscribe(this.localEventListener);
        this.ping(); // start pinger
    }
    RepetierServer.isSecure = function () {
        return window.location.protocol === 'https:';
    };
    RepetierServer.prototype.dispose = function () {
        this.disposed = true;
        this.printers.dispose();
    };
    RepetierServer.prototype.setPermissions = function (perm) {
        this.permissions = perm;
        this.canPrint = (perm & 1) == 1;
        this.canUpload = (perm & 2) == 2;
        this.canDelete = (perm & 4) == 4;
        this.canConfigure = (perm & 8) == 8;
        this.updateExtCommands();
    };
    RepetierServer.prototype.updateExtCommands = function () {
        var a = [];
        for (var idx in this.allExternalCommands) {
            var test = this.allExternalCommands[idx];
            if (test.permPrint && !this.canPrint)
                continue;
            if (test.permConfig && !this.canConfigure)
                return;
            if (test.permAdd && !this.canUpload)
                return;
            if (test.permDel && !this.canDelete)
                return;
            a.push(test);
        }
        this.filteredExternalCommands = a;
    };
    RepetierServer.prototype.connectUsingAPIKey = function (key) {
        //console.log("start connection with "+key);
        this.apikey = key;
        this.authenticated = true;
        this.frontend = false;
        this.startConnection();
    };
    RepetierServer.prototype.connectFrontend = function () {
        //console.log("start connection with "+key);
        this.apikey = "";
        this.authenticated = true;
        this.frontend = true;
        this.startConnection();
    };
    RepetierServer.prototype.connectUsingLogin = function (log, pw) {
        this.user.login = log;
        this.user.password = pw;
        this.apikey = undefined;
        this.authenticated = true;
        this.frontend = false;
        this.startConnection();
    };
    RepetierServer.prototype.fireLocalEvent = function (event, data, printer) {
        var e = new ServerEvent(event);
        if (printer) {
            e.printer = printer;
            e.slug = printer.slug;
        }
        else {
            e.printer = undefined;
            e.slug = undefined;
        }
        e.data = (data ? data : {});
        this.localEvents.next(e);
    };
    RepetierServer.prototype.hasFeature = function (f) {
        return (f & (this.update.features || 0)) == f;
    };
    ;
    // preload modal helper in case we are offline, we need this for message
    RepetierServer.prototype.updateImportant = function () {
        var _this = this;
        this.send("updateAvailable").then(function (r) {
            _this.update = r;
            if (r.currentVersion != _this.startVersion) {
                if (_this.startVersion !== '')
                    location.reload(true);
                _this.startVersion = r.currentVersion;
            }
            _this.hasFeature1 = _this.hasFeature(1);
        });
        this.loadSettings();
        this.updateFolders();
    };
    RepetierServer.prototype.close = function () {
        this.ws.onclose = function () {
        };
        this.ws.close();
    };
    RepetierServer.prototype.forceLogin = function () {
        /*    if ($location.path() != '/login')
         $rootScope.pathAfterLogin = $location.path();
         else
         $rootScope.pathAfterLogin = '/';
         if (User.autologin(Service)) return;
         $rootScope.$apply(function () {
         $location.path('/login');
         });*/
    };
    //$rootScope.$on('loginRequired', $rootScope.forceLogin);
    //$rootScope.$on('logout', User.logout);
    RepetierServer.prototype.sendRequest = function (request) {
        var _this = this;
        //console.log("send request ");console.log(request.action);
        var defer = new RSUtils_1.Deferred();
        request.callback_id = this.getCallbackId();
        if (request.action == "login") {
            this.ignoreErrorsTil = request.callback_id;
        }
        this.callbacks[request.callback_id] = {
            time: new Date(),
            cb: defer,
            request: request
        };
        if (!this.connected || (this.ws && this.ws.readyState != 1)) {
            var d = new RSUtils_1.Deferred();
            d.promise.then(function () {
                //request.session = User.getSession();
                if (!request.ws || request.ws !== _this.ws) {
                    delete (request.ws);
                    _this.ws.send(JSON.stringify(request));
                    request.ws = _this.ws;
                }
            });
            this.deflist.push(d);
        }
        else {
            this.ws.send(JSON.stringify(request));
            request.ws = this.ws;
        }
        return defer.promise;
    };
    RepetierServer.prototype.listener = function (data) {
        //console.log("listen");console.log(data);
        var messageObj = data;
        if (typeof (messageObj.session) != 'undefined' && this.session != messageObj.session) {
            this.session = messageObj.session;
        }
        // If an object exists with callback_id in our callbacks object, resolve it
        if (messageObj.callback_id < 0) {
            for (var _i = 0, _a = messageObj.data; _i < _a.length; _i++) {
                var evt = _a[_i];
                //console.log(evt);
                var e = new ServerEvent(evt.event);
                e.data = evt.data;
                e.printer = this.printers.printers[evt.printer];
                e.slug = evt.printer;
                this.serverEvents.next(e);
            }
        }
        else if (this.callbacks[messageObj.callback_id]) {
            var cb = this.callbacks[messageObj.callback_id];
            if (this.ignoreErrorsTil < 0 && this.ignoreErrorsTil > messageObj.callback_id)
                this.ignoreErrorsTil = -1;
            if (typeof (messageObj.data.permissionDenied) != 'undefined') {
                if (this.ignoreErrorsTil >= 0 && messageObj.callback_id < this.ignoreErrorsTil) {
                    cb.cb.reject();
                    delete this.callbacks[messageObj.callback_id];
                }
                else {
                    cb.cb.reject();
                    delete this.callbacks[messageObj.callback_id];
                    this.forceLogin();
                }
                return;
            }
            else if (typeof (messageObj.data.alert) != 'undefined') {
                cb.cb.reject();
                delete this.callbacks[messageObj.callback_id];
                //$rootScope.alert = messageObj.data.alert;
                return;
            }
            //console.log("listen resolve:");console.log(data);
            cb.cb.resolve(messageObj.data);
            //console.log("parsed");
            delete this.callbacks[messageObj.callback_id];
        }
        //console.log("tick");
        //this.app.tick();
    };
    // This creates a new callback ID for a request
    RepetierServer.prototype.getCallbackId = function () {
        this.currentCallbackId += 1;
        if (this.currentCallbackId > 10000) {
            this.currentCallbackId = 0;
        }
        return this.currentCallbackId;
    };
    RepetierServer.prototype.send = function (command, data, slug) {
        if (!slug)
            slug = this.printerSlug;
        if (!data)
            data = {};
        return this.sendRequest({ action: command, data: data, printer: slug });
    };
    RepetierServer.prototype.selectPrinter = function (slug) {
        this.printerSlug = slug;
    };
    RepetierServer.prototype.extendPing = function (timeout) {
        this.send("extendPing", { timeout: timeout });
    };
    RepetierServer.prototype.serverSettingsChanged = function () {
        var modified = {};
        for (var key in this.settings) {
            if (!this.settings.hasOwnProperty(key))
                continue;
            var val = this.settings[key];
            if (!RSUtils_1.RSUtils.equals(val, this._originalSettings[key])) {
                modified[key] = val;
            }
        }
        if (RSUtils_1.RSUtils.isEmptyObject(modified)) {
            return;
        }
        return this.send("updateSettings", modified);
    };
    RepetierServer.prototype.loadSettings = function () {
        var _this = this;
        this.send("getSettings", {}).then(function (r) {
            RSUtils_1.RSUtils.extend(_this.settings, r);
            _this._originalSettings = RSUtils_1.RSUtils.copyObject(_this.settings);
        });
    };
    RepetierServer.prototype.networkInterfaces = function () {
        return this.send("networkInterfaces");
    };
    RepetierServer.prototype.runExternalCommand = function (cmd) {
        return this.send("runExternalCommand", { id: cmd.id });
    };
    RepetierServer.prototype.updateFolders = function () {
        var _this = this;
        this.send("getFolders", {}).then(function (data) {
            _this.folders = [];
            for (var f in data.folders) {
                _this.folders.push(RSUtils_2.SerializationHelper.convert(new ServerFolder(), data.folders[f]));
            }
        });
    };
    RepetierServer = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [core_1.ApplicationRef, http_1.Http])
    ], RepetierServer);
    return RepetierServer;
})();
exports.RepetierServer = RepetierServer;
//# sourceMappingURL=RepetierServer.js.map