/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from '@angular/core';
import {NgRedux} from 'ng2-redux';
import {RepetierState} from '../state/RepetierState';
import {Language} from "../state/Language";
import {ExternalCommand} from "../state/ExternalCommand";
import {ConnectionHandler} from "../ConnectionHandler";
import {RSUtils, SerializationHelper} from "../RSUtils";
import {ServerFolderState, INITIAL_SERVER_FOLDER_STATE} from "../state/ServerState";

@Injectable()
export class ServerAction {

    static SET_LANGUAGES: string = 'SERVER_SET_LANGUAGES';
    static SET_EXTERNAL_COMMANDS: string = 'SERVER_SET_EXTERNAL_COMMANDS';
    static SET_ALL_EXTERNAL_COMMANDS: string = 'SERVER_SET_ALL_EXTERNAL_COMMANDS';
    static SET_GLOBAL: string = 'SERVER_SET_GLOBAL';
    static SET_SETTINGS: string = 'SERVER_SET_SETTINGS';
    static INIT_SETTING: string = 'SERVER_INIT_SETTING';
    static UPDATE_SETTINGS: string = 'SERVER_UPDATE_SETTING';
    static SET_SERVER_FOLDERS: string = 'SERVER_SET_SERVER_FOLDERS';

    private openInits: any[] = [];

    constructor(private redux: NgRedux<RepetierState>, private connectionHandler: ConnectionHandler) {
        this.openInits.push({
            key: "pricing", data: {
                currency: 'EUR',
                perPrint: 5.0,
                perHour: 0.3,
                filaments: [{u: RSUtils.createAlphaID(10), n: 'PLA', p: 25.0, w: 1.25}],
                map: {}
            }
        });
    }

    setLanguages(languages: Language[]): void {
        this.redux.dispatch({type: ServerAction.SET_LANGUAGES, payload: languages});
    }

    setExternalCommands(commands: ExternalCommand[]): void {
        this.redux.dispatch({type: ServerAction.SET_EXTERNAL_COMMANDS, payload: commands});
    }

    setAllExternalCommands(commands: ExternalCommand[]): void {
        this.redux.dispatch({type: ServerAction.SET_ALL_EXTERNAL_COMMANDS, payload: commands});
    }

    setGlobalVar(key: string, data: any): void {
        this.redux.dispatch({type: ServerAction.SET_GLOBAL, payload: {key: key, data: data}});
    }

    setSettings(data: {[key: string]: any}): void {
        if (RSUtils.newChanges(this.redux.getState().server.settings, data)) {
            this.redux.dispatch({type: ServerAction.SET_SETTINGS, payload: data});
        }
    }

    initSetting(key: string, data: any): void {
        if (this.redux.getState().server.settingsLoaded && this.redux.getState().server.settings[key] === undefined) {
            let a: {[key: string]: any} = {};
            a[key] = data;
            this.setSettings(a); // immediate update for issuing device
            this.connectionHandler.send("updateSettings", a);
        } else if (!this.redux.getState().server.settingsLoaded) {
            this.openInits.push({key: key, data: data});
        }
    }

    updateSetting(key: string, data: any): Promise<any> {
        if (this.redux.getState().server.settingsLoaded && !RSUtils.equals(this.redux.getState().server.settings[key], data)) {
            let a: {[key: string]: any} = {};
            a[key] = data;
            this.setSettings(a); // immediate update for issuing device
            return this.connectionHandler.send("updateSettings", a);
        }
        return Promise.resolve();
    }

    loadSettings() {
        this.connectionHandler.send("getSettings", {}).then((r) => {
            this.setSettings(r);
            for (let i = 0; i < this.openInits.length; i++) {
                this.initSetting(this.openInits[i].key, this.openInits[i].data);
            }
            this.openInits = [];
        });
    }

    runExternalCommand(cmd: ExternalCommand): Promise<any> {
        return this.connectionHandler.send("runExternalCommand", {id: cmd.id});
    }

    updateFolders() {
        this.connectionHandler.send("getFolders", {}).then((data) => {
            let folders:ServerFolderState[] = [];
            for (let f in data.folders) {
                if(data.folders.hasOwnProperty(f)) {
                    folders.push(SerializationHelper.convert(Object.assign({}, INITIAL_SERVER_FOLDER_STATE), data.folders[f]));
                }
            }
            this.redux.dispatch({type:ServerAction.SET_SERVER_FOLDERS,payload:folders});
        })
    }
}
