/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {PrintersState, INITIAL_PRINTERS_STATE} from "../state/PrintersState";
import {PrinterState} from "../state/PrinterState";
import {PrintersAction} from "../actions/PrintersAction";
import {PrinterReducer} from "./PrinterReducer";

export function PrintersReducer(state: PrintersState = INITIAL_PRINTERS_STATE, action:any) :PrintersState {
    function sortPrinters(printers:{[key: string]: PrinterState}) {
        let sortedPrinters:string[] = [];
        for(let p in printers) {
            if(printers.hasOwnProperty(p))
                sortedPrinters.push(p);
        }
        sortedPrinters.sort(function (aSlug:string, bSlug:string) {
            let a:PrinterState = printers[aSlug];
            let b:PrinterState = printers[bSlug];
            // ['-online', '-active', 'name']);//r;
            if (a.online < b.online) return 1;
            if (a.online > b.online) return -1;
            if (a.active && !b.active) return -1;
            if (!a.active && b.active) return 1;
            let o1 = a.name.toLowerCase();
            let o2 = b.name.toLowerCase();
            if (o1 < o2) return -1;
            if (o1 > o2) return 1;
            return 0;
        });
        return sortedPrinters;
    }
    function updateActive(state:PrintersState):PrintersState {
        if(state.sortedPrinters.length == 0) {
            state.active = undefined;
            return state;
        }
        if (state.active === undefined) {
            state.active = state.sortedPrinters[0];
        }
        if (state.autoselectActive && state.printers[state.active].online != 1 && state.printers[state.sortedPrinters[0]].online == 1) {
            state.active = state.sortedPrinters[0];
        }
        return state;
    }

    let parts:string[] = action.type.split("/");
    if(parts.length > 2 && parts[0]=="printer") { // aggregated type with pointer - move slug dependend actions to printer reducer
        let slug = parts[1];
        if(slug.length == 0)
            slug = state.active;
        if(!slug) {
            console.error("PrintersReducer called for printer without printer slug set or active slug");
            return state;
        }
        if(!state.printers[slug]) {
            console.error("PrintersReducer called for printer with invalid/unknown slug "+slug);
            return state;
        }
        action.type = parts.slice(2).join("/"); // remove slug part
        let modifiedPrinters:{[key:string]:PrinterState} = {};
        modifiedPrinters[slug] = Object.assign(PrinterReducer(state.printers[slug],action));
        return Object.assign({},state,{printers:Object.assign({},state.printers,modifiedPrinters)});
    } else {
        switch (action.type) {
            case PrintersAction.SET_ACTIVE:
                return Object.assign({}, state, {active: action.payload.slug,autoselectActive:false});
            case PrintersAction.CREATE_PRINTER: {
                let printers:{[key: string]: PrinterState} = Object.assign({},state.printers);
                printers[action.payload.slug] = action.payload;
                return updateActive(Object.assign({}, state, {printers: printers,sortedPrinters:sortPrinters(printers)}));
            }
            case PrintersAction.DESTROY_PRINTER: {
                let printers = Object.assign({}, state.printers);
                delete printers[action.payload.slug];
                return updateActive(Object.assign({}, state, {printers: printers, sortedPrinters: sortPrinters(printers)}));
            }
            case PrintersAction.SET_AUTOSELECT: {
                return updateActive(Object.assign({},state,{autoselectActive:action.payload}));
            }
            case PrintersAction.SET_STATE_TIME:
                return Object.assign({},state,{stateTime:action.payload});
            default:
                return state;
        }
    }
}
