/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var nextId = 0;
var RSGauge = (function () {
    function RSGauge() {
        this.width = 50;
        this.height = 50;
        this.angle = 359.9; // Max angle
        this.angleOffset = 0; // offset from 12:00
        this.rangeWidth = 6;
        this.valueWidth = 4;
        this.showTarget = false;
        this.target = 0;
        this.showTicks = true;
        this.tickDistance = 18;
        this.value = 90;
        this.minValue = 0;
        this.maxValue = 100;
        this.divider = false;
        this.type = "primary";
        this.bgClass = "svg-text-primary";
        this.fgClass = "svg-primary";
        this.radius = 25;
        this.backArc = "[]";
        this.frontArc = "[]";
        this.dividerPath = "[]";
        this.ticks = "";
        this.ticks2 = "";
    }
    RSGauge.prototype.polarToCartesian = function (centerX, centerY, radius, angleInDegrees) {
        var angleInRadians = (angleInDegrees + this.angleOffset) * Math.PI / 180.0;
        return {
            x: centerX + radius * Math.sin(angleInRadians),
            y: centerY - radius * Math.cos(angleInRadians)
        };
    };
    RSGauge.prototype.updateGauge = function () {
        var ang = 360 * (this.value - this.minValue) / (this.maxValue - this.minValue);
        var mw = Math.max(this.rangeWidth, this.valueWidth);
        this.radius = (this.width - mw) / 2;
        var center = this.width / 2;
        var start = this.polarToCartesian(center, center, this.radius, 0);
        var end = this.polarToCartesian(center, center, this.radius, this.angle);
        var end2 = this.polarToCartesian(center, center, this.radius, ang);
        var arcSweep = this.angle <= 180 ? "0" : "1";
        this.backArc = [
            "M", start.x, start.y,
            "A", this.radius, this.radius, 0, arcSweep, 1, end.x, end.y
        ].join(" ");
        arcSweep = ang <= 180 ? "0" : "1";
        this.frontArc = [
            "M", start.x, start.y,
            "A", this.radius, this.radius, 0, arcSweep, 1, end2.x, end2.y
        ].join(" ");
        this.ticks = "";
        this.ticks2 = "";
        if (this.showTicks) {
            var alpha = 0;
            var n = Math.round(this.angle / this.tickDistance);
            var a = [];
            var a2 = [];
            for (var i = 0; i < n; i++) {
                var s = this.polarToCartesian(center, center, this.radius - 0.5 * this.rangeWidth, alpha);
                var e = this.polarToCartesian(center, center, this.radius + 0.5 * this.rangeWidth, alpha);
                a = a.concat("M", s.x, s.y, "L", e.x, e.y);
                if (alpha < this.value) {
                    s = this.polarToCartesian(center, center, this.radius - 0.5 * this.valueWidth, alpha);
                    e = this.polarToCartesian(center, center, this.radius + 0.5 * this.valueWidth, alpha);
                    a2 = a2.concat("M", s.x, s.y, "L", e.x, e.y);
                }
                alpha += this.tickDistance;
            }
            this.ticks = a.join(" ");
            this.ticks2 = a2.join(" ");
        }
        if (this.divider) {
            var s = this.polarToCartesian(center, center, this.radius - mw, 90);
            var e = this.polarToCartesian(center, center, this.radius - mw, 270);
            this.dividerPath = ["M", s.x, s.y, "L", e.x, e.y].join(" ");
        }
    };
    RSGauge.prototype.ngOnInit = function () {
        this.updateGauge();
    };
    RSGauge.prototype.ngDoCheck = function () {
        this.updateGauge();
    };
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "width", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "height", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "angle", void 0);
    __decorate([
        // Max angle
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "angleOffset", void 0);
    __decorate([
        // offset from 12:00
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "rangeWidth", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "valueWidth", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], RSGauge.prototype, "showTarget", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "target", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], RSGauge.prototype, "showTicks", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "tickDistance", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "value", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "minValue", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSGauge.prototype, "maxValue", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], RSGauge.prototype, "divider", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], RSGauge.prototype, "type", void 0);
    RSGauge = __decorate([
        core_1.Component({
            selector: 'rs-gauge',
            template: "\n    <div class=\"g\" [ngStyle]=\"{width:width+'px',height:height+'px'}\"><div class=\"g2\"><ng-content></ng-content></div></div>\n    <svg [attr.width]=\"width\" [attr.height]=\"height\" preserveAspectRatio=\"xMidYMid meet\">\n    <path [attr.d]=\"backArc\" fill=\"none\" stroke=\"#444444\" [attr.stroke-width]=\"rangeWidth\" />\n    <path [attr.d]=\"frontArc\" fill=\"none\" [attr.class]=\"fgClass\" [attr.stroke-width]=\"valueWidth\" />\n    <path *ngIf=\"showTicks\" [attr.d]=\"ticks\" fill=\"none\" [attr.class]=\"bgClass\" stroke-width=\"2\" />\n    <!--<path *ngIf=\"showTicks\" [attr.d]=\"ticks2\" fill=\"none\" stroke=\"#000000\" stroke-width=\"2\" />-->\n    <path *ngIf=\"divider\" [attr.d]=\"dividerPath\" [attr.class]=\"fgClass\" stroke-width=\"2\" stroke-dasharray=\"5,5\" />\n    </svg>\n    ",
            host: {
                class: 'gauge'
            }
        }), 
        __metadata('design:paramtypes', [])
    ], RSGauge);
    return RSGauge;
})();
exports.RSGauge = RSGauge;
//# sourceMappingURL=gauge.component.js.map