/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var nextId = 0;
var RSSwitch = (function () {
    function RSSwitch(elem) {
        this.elem = elem;
        this.disabled = false;
        this.trueValue = true;
        this.falseValue = false;
        this.checkedChange = new core_1.EventEmitter();
        this.initialized = false;
        this._value = false;
        this.id = "rsswitch-" + ++nextId;
    }
    RSSwitch.prototype.toBool = function (v) {
        return (v != this.falseValue);
    };
    RSSwitch.prototype.toValue = function (v) {
        if (v)
            return this.trueValue;
        return this.falseValue;
    };
    Object.defineProperty(RSSwitch.prototype, "checked", {
        get: function () {
            return this.toValue(this._value);
        },
        set: function (val) {
            this.value = this.toBool(val);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RSSwitch.prototype, "inputId", {
        get: function () {
            return "input-" + this.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RSSwitch.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (checked) {
            if (checked != this._value) {
                this._value = checked;
                // Only fire a change event if this isn't the first time the checked property is ever set.
                if (this.initialized) {
                    this.checkedChange.emit(this.toValue(this._value));
                }
            }
            this.initialized = true;
        },
        enumerable: true,
        configurable: true
    });
    RSSwitch.prototype.toggle = function () {
        this.value = !this._value;
    };
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], RSSwitch.prototype, "disabled", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], RSSwitch.prototype, "trueValue", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], RSSwitch.prototype, "falseValue", void 0);
    __decorate([
        core_1.Output(), 
        __metadata('design:type', core_1.EventEmitter)
    ], RSSwitch.prototype, "checkedChange", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], RSSwitch.prototype, "checked", null);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], RSSwitch.prototype, "id", void 0);
    RSSwitch = __decorate([
        core_1.Component({
            selector: 'rs-switch',
            template: "\n    <label><div class=\"shape\" [ngClass]=\"{disabled:disabled}\" >\n    <input [id]=\"inputId\" [(ngModel)]=\"value\" type=\"checkbox\" [disabled]=\"disabled\"/>\n    <div class=\"button\"></div>\n    <div class=\"stripe\"></div>\n    </div>\n<ng-content></ng-content></label>",
            host: {
                class: 'switch'
            }
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef])
    ], RSSwitch);
    return RSSwitch;
})();
exports.RSSwitch = RSSwitch;
var RSRadio = (function () {
    function RSRadio(elem) {
        this.elem = elem;
        this.disabled = false;
        this.on = undefined;
        this.valueChange = new core_1.EventEmitter();
        this.initialized = false;
        this._value = false;
    }
    Object.defineProperty(RSRadio.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (v) {
            this._value = v;
            this.valueChange.emit(v);
        },
        enumerable: true,
        configurable: true
    });
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], RSRadio.prototype, "disabled", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], RSRadio.prototype, "on", void 0);
    __decorate([
        core_1.Output(), 
        __metadata('design:type', core_1.EventEmitter)
    ], RSRadio.prototype, "valueChange", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], RSRadio.prototype, "value", null);
    RSRadio = __decorate([
        core_1.Component({
            selector: 'rs-radio',
            template: "\n    <div class=\"on\" *ngIf=\"on==_value\"></div>",
            host: {
                class: 'radio',
                "(click)": "value=on"
            }
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef])
    ], RSRadio);
    return RSRadio;
})();
exports.RSRadio = RSRadio;
//# sourceMappingURL=switch.component.js.map