/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, Output, ElementRef, EventEmitter} from '@angular/core';

let nextId = 0;

@Component({
    selector: 'rs-switch',
    template: `
    <input [id]="inputId" [(ngModel)]="value" type="checkbox" style="display:none"/>
    <label (rsclick)="toggle()"><div class="shape" [ngClass]="{disabled:disabled}" >
    <div class="button" [class.on]="_value"></div>
    <div class="stripe" [class.on]="_value"></div>
    </div>
<ng-content></ng-content></label>`,
    host: {
        class: 'switch'
    }
})

export class RSSwitch {
    @Input() disabled: boolean = false;
    @Input() trueValue: any = true;
    @Input() falseValue: any = false;
    @Output() checkedChange: EventEmitter<any> = new EventEmitter<any>();

    //private initialized: boolean = false;
    private _value: boolean = false;

    constructor(public elem: ElementRef) {
    }

    private toBool(v: any): boolean {
        return (v != this.falseValue);
    }

    private toValue(v: boolean): any {
        if (v)
            return this.trueValue;
        return this.falseValue;
    }

    @Input() get checked() {
        return this.toValue(this._value);
    }

    set checked(val: any) {
        if(val === undefined)
            return;
        this.value = this.toBool(val);
    }

    @Input() id: string = `rsswitch-${++nextId}`;

    get inputId(): string {
        return `input-${this.id}`;
    }

    get value() {
        return this._value;
    }

    set value(checked: boolean) {
        if(checked == undefined)
            return;
        if (checked != this._value) {
            this._value = checked;
            // Only fire a change event if this isn't the first time the checked property is ever set.
            //if (this.initialized) {
            //    this.checkedChange.emit(this.toValue(this._value));
            //}
        }

        //this.initialized = true;
    }

    toggle() {
        if(this.disabled)
            return;
        this._value = !this._value;
        this.checkedChange.emit(this.toValue(this._value));
    }

}

@Component({
    selector: 'rs-radio',
    template: `
    <div (rsclick)="value=on"><div class="on" *ngIf="on==_value"></div></div>`
    ,host: {
        "class": 'radio'
    }
})
export class RSRadio {
    @Input() disabled: boolean = false;
    @Input() on: any = undefined;

    @Output() valueChange: EventEmitter<any> = new EventEmitter<any>();

    private initialized: boolean = false;
    private _value: any = false;

    constructor(public elem: ElementRef) {
    }

    @Input() get value() {
        return this._value;
    }

    set value(v) {
        this._value = v;
        this.valueChange.emit(v);
    }
}