/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
var core_1 = require('@angular/core');
var RSBinding_1 = require("../RSBinding");
var RSTempGraph = (function () {
    function RSTempGraph(server, elementRef) {
        this.server = server;
        this.elementRef = elementRef;
        this.extruder = undefined;
        this.ok = false;
        this.width = 1;
        this.height = 1;
        this.minTemp = 0;
        this.maxTemp = 300;
        this.lastTime = 0;
        this.tempLegend = [];
        this.outputLegend = [];
        this.timeLegend = [];
        this.tickLength = 5;
    }
    RSTempGraph.prototype.timeToX = function (t) {
        return this.left + (t - this.minTime) * (this.right - this.left) / (this.maxTime - this.minTime);
    };
    RSTempGraph.prototype.tempToY = function (t) {
        return this.bottom - (t - this.minTemp) * (this.bottom - this.top) / (this.maxTemp - this.minTemp);
    };
    RSTempGraph.prototype.outputToY = function (t) {
        return this.bottom - t * (this.bottom - this.top) / 100;
    };
    RSTempGraph.prototype.twoDigits = function (a) {
        a = a.toFixed(0);
        if (a.length == 1)
            return "0" + a;
        return a;
    };
    RSTempGraph.prototype.updateGraph = function () {
        if (this.extruder === undefined) {
            this.ok = false;
            return;
        }
        this.graph = [];
        if (this.extruder < 0) {
            if (this.server.active && this.server.active.heatedBed && this.server.active.heatedBed.tempHistory)
                this.graph = this.server.active.heatedBed.tempHistory;
        }
        else {
            if (this.server.active && this.server.active.extruders && this.server.active.extruders.length > this.extruder && this.server.active.extruders[this.extruder].tempHistory)
                this.graph = this.server.active.extruders[this.extruder].tempHistory;
        }
        if (this.graph.length == 0) {
            this.ok = false;
            return;
        }
        if (this.lastTime == this.graph[this.graph.length - 1].time)
            return; // no new data
        this.minTemp = 99999;
        this.maxTemp = -99999;
        this.maxTime = this.graph[this.graph.length - 1].time;
        this.minTime = this.maxTime - 1000 * 5 * 60; // 5 minutes display
        for (var idx in this.graph) {
            var act = this.graph[idx];
            if (act.temp < this.minTemp)
                this.minTemp = act.temp;
            if (act.temp > this.maxTemp)
                this.maxTemp = act.temp;
            if (act.tempSet < this.minTemp)
                this.minTemp = act.tempSet;
            if (act.tempSet > this.maxTemp)
                this.maxTemp = act.tempSet;
        }
        if (this.minTemp < 0)
            this.minTemp = 0; // minimum is 0
        this.minTemp = Math.max(0, Math.round((this.minTemp - 12) / 10) * 10) - 5;
        this.maxTemp = Math.max(this.minTemp + 20, Math.round((this.maxTemp + 12) / 10) * 10);
        var w = this.elementRef.nativeElement.offsetWidth;
        var h = this.elementRef.nativeElement.parentElement.offsetHeight;
        this.width = w;
        this.height = h;
        var showOutLegend = w > 200;
        this.left = 50;
        this.right = showOutLegend ? w - 45 : w - this.tickLength;
        this.bottom = h - 25;
        this.top = 10;
        var ticks = [];
        //console.log("dim "+w+" x "+h);
        this.axis = ["M", this.left, this.top, "L", this.left, this.bottom, "L", this.right, this.bottom, "L", this.right, this.top,
            "L", this.left, this.top
        ].join(" ");
        var grid = [];
        var actTime = Math.round(this.minTime / 60000) * 60000;
        this.timeLegend = [];
        do {
            if (actTime > this.minTime && actTime < this.maxTime) {
                var xt = this.timeToX(actTime);
                grid.push("M");
                grid.push(xt);
                grid.push(this.bottom);
                grid.push("L");
                grid.push(xt);
                grid.push(this.top);
                ticks.push("M");
                ticks.push(xt);
                ticks.push(this.bottom);
                ticks.push("L");
                ticks.push(xt);
                ticks.push(this.bottom + this.tickLength);
                var d = new Date(actTime);
                this.timeLegend.push({ x: xt, y: this.height - 5, t: "" + d.getHours() + ":" + this.twoDigits(d.getMinutes()) });
            }
            actTime += 60000;
        } while (actTime < this.maxTime);
        var actTemp = this.minTemp + 5;
        do {
            var yt = this.tempToY(actTemp);
            if (yt < this.bottom && yt > this.top) {
                grid.push("M");
                grid.push(this.left);
                grid.push(yt);
                grid.push("L");
                grid.push(this.right);
                grid.push(yt);
                ticks.push("M");
                ticks.push(this.left);
                ticks.push(yt);
                ticks.push("L");
                ticks.push(this.left - this.tickLength);
                ticks.push(yt);
            }
            actTemp += 10;
        } while (actTemp < this.maxTemp);
        // Output ticks
        this.outputLegend = [];
        for (var i = 0; i <= 100; i += 25) {
            var yt = this.outputToY(i);
            ticks.push("M");
            ticks.push(this.right);
            ticks.push(yt);
            ticks.push("L");
            ticks.push(this.right + this.tickLength);
            ticks.push(yt);
            if (showOutLegend) {
                this.outputLegend.push({ x: this.right + 5 + this.tickLength, y: yt, t: "" + i + "%" });
            }
        }
        this.grid = grid.join(" ");
        this.ticks = ticks.join(" ");
        var temp = [];
        var tempSet = [];
        var out = [];
        var lastX = 0;
        for (var idx in this.graph) {
            var t = this.graph[idx];
            var xp = this.timeToX(t.time);
            if (lastX > xp) {
                //console.log("error x "+idx + " Last "+lastX+" act "+xp);console.log(this.graph);
                continue;
            }
            lastX = xp;
            if (xp >= this.left) {
                if (temp.length == 0) {
                    temp.push("M");
                    tempSet.push("M");
                    out.push("M");
                }
                else {
                    temp.push("L");
                    tempSet.push("L");
                    out.push("L");
                }
                temp.push(xp);
                tempSet.push(xp);
                out.push(xp);
                temp.push(this.tempToY(t.temp));
                tempSet.push(this.tempToY(t.tempSet));
                out.push(this.outputToY(t.output));
            }
        }
        out.push("L");
        out.push(xp);
        out.push(this.bottom);
        out.push("L");
        out.push(out[1]);
        out.push(this.bottom);
        out.push("L");
        out.push(out[1]);
        out.push(out[2]);
        this.curTemp = temp.join(" ");
        this.setTemp = tempSet.join(" ");
        this.output = out.join(" ");
        // write legends
        this.tempLegend = [];
        var dy = Math.abs(this.tempToY(90) - this.tempToY(100));
        var every = Math.floor(15 / dy) + 1;
        //console.log("every "+every);
        var actTemp = this.minTemp + 5;
        while (actTemp <= this.maxTemp) {
            this.tempLegend.push({
                x: this.left - this.tickLength - 5,
                y: this.tempToY(actTemp),
                t: "" + actTemp + "°C"
            });
            actTemp += every * 10;
        }
        this.ok = true;
        this.lastTime = this.graph[this.graph.length - 1].time;
        //console.log(this);
    };
    RSTempGraph.prototype.ngOnInit = function () {
        this.updateGraph();
    };
    RSTempGraph.prototype.ngDoCheck = function () {
        this.updateGraph();
    };
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSTempGraph.prototype, "extruder", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], RSTempGraph.prototype, "tickLength", void 0);
    RSTempGraph = __decorate([
        core_1.Component({
            selector: 'rs-temp-graph',
            template: "\n<svg  *ngIf=\"ok\" [attr.width]=\"width\" [attr.height]=\"height\" preserveAspectRatio=\"xMidYMid meet\">\n    <path [attr.d]=\"grid\" class=\"grid\"/>\n    <path [attr.d]=\"axis\" class=\"axis\"/>\n    <path [attr.d]=\"ticks\" class=\"ticks\"/>\n    <path [attr.d]=\"output\" class=\"output\"/>\n    <path [attr.d]=\"setTemp\" class=\"tempSet\"/>\n    <path [attr.d]=\"curTemp\" class=\"temp\"/>\n    <text *ngFor=\"let t of tempLegend\" [attr.x]=\"t.x\" [attr.y]=\"t.y\" class=\"tempAxisText\">{{t.t}}</text>\n    <text *ngFor=\"let t of timeLegend\" [attr.x]=\"t.x\" [attr.y]=\"t.y\" class=\"timeAxisText\">{{t.t}}</text>\n    <text *ngFor=\"let t of outputLegend\" [attr.x]=\"t.x\" [attr.y]=\"t.y\" class=\"outputAxisText\">{{t.t}}</text>\n    </svg>\n    ",
            directives: [],
            host: {
                class: 'graph'
            }
        }),
        __param(1, core_1.Inject(core_1.ElementRef)), 
        __metadata('design:paramtypes', [RSBinding_1.RepetierServer, core_1.ElementRef])
    ], RSTempGraph);
    return RSTempGraph;
})();
exports.RSTempGraph = RSTempGraph;
//# sourceMappingURL=temp-graph.component.js.map